import sys, os
import numpy as np
from scipy.optimize import curve_fit
from matplotlib import pyplot as plt
import BioNetInput, BioNetMeasurement, BioNetTrajectory

def expFit(t, tau, y0, yf):

	return (y0 - yf) * np.exp(- t / tau) + yf

def calculatePlanes(frame, width, numNodes):

	lPlane = [[],[],[]]
	rPlane = [[],[],[]]

	# Calculate the dimensions
	spans = frame.calculateSpan()	
	for j in range(3):
		for i in range(numNodes):

			# Forces
			if frame.pos[i][j] < spans[0][j] + width:
				lPlane[j].append(i)

			elif frame.pos[i][j] > spans[1][j] - width:
				rPlane[j].append(i)


	return lPlane[0], rPlane[0], lPlane[1], rPlane[1], lPlane[2], rPlane[2]

def calculatePlanes2(frame, numNodes):

	lPlane = [[],[],[]]
	rPlane = [[],[],[]]

	# Calculate the dimensions
	xList = [(i, frame.pos[i][0]) for i in range(numNodes)]
	yList = [(i, frame.pos[i][1]) for i in range(numNodes)]
	zList = [(i, frame.pos[i][2]) for i in range(numNodes)]

	xListSorted = sorted(xList,key=lambda tup: tup[1])
	yListSorted = sorted(yList,key=lambda tup: tup[1])
	zListSorted = sorted(zList,key=lambda tup: tup[1])

	# Top 10% of particles are pulled on each side
	pullNumNodes = int(numNodes / 10.0)
	lPlane[0] = [tup[0] for tup in xListSorted[:pullNumNodes]]
	rPlane[0] = [tup[0] for tup in xListSorted[-pullNumNodes:]]
	lPlane[1] = [tup[0] for tup in yListSorted[:pullNumNodes]]
	rPlane[1] = [tup[0] for tup in yListSorted[-pullNumNodes:]]
	lPlane[2] = [tup[0] for tup in zListSorted[:pullNumNodes]]
	rPlane[2] = [tup[0] for tup in zListSorted[-pullNumNodes:]]
	return lPlane[0], rPlane[0], lPlane[1], rPlane[1], lPlane[2], rPlane[2]


SMALL = 6
MEDIUM = 8
LARGE = 10

#ffs = [1.00, 0.91,0.72]

ffs = [1.00,0.91,0.72]

radius = 3.3
numRuns = 3

coordFig = plt.figure(figsize=(3.2,3.2))
energyFig = plt.figure(figsize=(3.2,3.2))
xLFig = plt.figure(figsize=(3.2,3.2))

fIndex = 0
for ff in ffs:

	dFname = "../results/BSAHydrogel_ff%.3f_Kinetics_Data.dat" % (ff)
	fout = open(dFname, "w")
	fout.write("Run, xtau, xtauerr, xf,xferr,ytau,ytauerr,yf,yferr,ztau,ztauerr,zf,zferr\n")
	for nr in range(numRuns):

		# Get initial script
		iFname = "../input/BSAHydrogel_ff%.3f_Kinetics_Run%d.xml" % (ff,nr)
		inp = BioNetInput.BioNetInput()
		inp.load(iFname)


		# Get trajectories
		tFname = "../output/BSAHydrogel_ff%.3f_Kinetics_Run%d.bntrj" % (ff,nr)
		traj = BioNetTrajectory.BioNetTrajectory()
		traj.load(tFname, rate=10)

		mFname = "../output/BSAHydrogel_ff%.3f_Kinetics_Run%d.bnms" % (ff,nr)
		meas = BioNetMeasurement.BioNetMeasurement()
		meas.load(mFname, rate=10)

		# Plot energy traces
		plt.figure(energyFig.number)
		plt.semilogy(meas.time[:end], meas.elasticEnergy[:end], "xC%d" % (nr), label="Run %d" % (nr))
		plt.semilogy(meas.time[:end], meas.stericEnergy[:end], "oC%d" % (nr))

		# Plot coordination traces
		plt.figure(coordFig.number)
		plt.plot(meas.time[:end], meas.coordination[:end], "xC%d" % (nr), label="Run %d" % (nr))

		# Calculate widths
		forceSegment = inp.box.dimension[0] / 6.0

		# Get widths over time
		frIndex = 0
		xLs = np.array([0.0 for f in traj.frame])
		yLs = np.array([0.0 for f in traj.frame])
		zLs = np.array([0.0 for f in traj.frame])
		for f in traj.frame:

			xleft, xright, yleft, yright, zleft, zright = calculatePlanes2(f, inp.numNodes)
			numxLeft = len(xleft)
			numxRight = len(xright)
			numyLeft = len(yleft)
			numyRight = len(yright)
			numzLeft = len(zleft)
			numzRight = len(zright)

			xlCentroid = np.array([0.0,0.0,0.0])
			xrCentroid = np.array([0.0,0.0,0.0])
			ylCentroid = np.array([0.0,0.0,0.0])
			yrCentroid = np.array([0.0,0.0,0.0])
			zlCentroid = np.array([0.0,0.0,0.0])
			zrCentroid = np.array([0.0,0.0,0.0])

			for i in xleft:
				xlCentroid += f.pos[i]
			xlCentroid /= numxLeft

			for i in xright:
				xrCentroid += f.pos[i]
			xrCentroid /= numxRight

			for i in yleft:
				ylCentroid += f.pos[i]
			ylCentroid /= numyLeft

			for i in yright:
				yrCentroid += f.pos[i]
			yrCentroid /= numyRight

			for i in zleft:
				zlCentroid += f.pos[i]
			zlCentroid /= numzLeft

			for i in zright:
				zrCentroid += f.pos[i]
			zrCentroid /= numzRight

			xLs[frIndex] = xrCentroid[0] - xlCentroid[0]
			yLs[frIndex] = yrCentroid[1] - ylCentroid[1]
			zLs[frIndex] = zrCentroid[2] - zlCentroid[2]

			frIndex += 1

		plt.figure(xLFig.number)
		plt.plot(meas.time[:end], xLs[:end], "xC%d" % (nr), label="Run %d" % (nr))
		plt.plot(meas.time[:end], yLs[:end], "oC%d" % (nr))
		plt.plot(meas.time[:end], zLs[:end], "^C%d" % (nr))

		"""
		# Fits
		fout.write("%d" % (nr))
		if(ff != 1.00):

			# Exponential decay
			guess = [5000, xLs[0], xLs[0] * 0.8]
			popt, pcov = curve_fit(expFit, meas.time, xLs, guess)
			plt.plot(meas.time, expFit(meas.time, *popt), "--C%d" % (nr))
			fout.write(",%f, %f, %f,%f" % (popt[0], np.sqrt(pcov[0][0]), popt[2], np.sqrt(pcov[2][2])))

			guess = [5000, yLs[0], yLs[0] * 0.8]
			popt, pcov = curve_fit(expFit, meas.time, yLs, guess)
			plt.plot(meas.time, expFit(meas.time, *popt), "--C%d" % (nr))
			fout.write(",%f, %f, %f,%f" % (popt[0], np.sqrt(pcov[0][0]), popt[2], np.sqrt(pcov[2][2])))

			guess = [5000, zLs[0], zLs[0] * 0.8]
			popt, pcov = curve_fit(expFit, meas.time, zLs, guess)
			plt.plot(meas.time, expFit(meas.time, *popt), "--C%d" % (nr))
			fout.write(",%f, %f, %f,%f" % (popt[0], np.sqrt(pcov[0][0]), popt[2], np.sqrt(pcov[2][2])))

		else:
			# Just average the traces (and assume 0.0 decay time)
			fout.write(",%f, %f, %f,%f" % (0.0, 0.0, np.mean(xLs), np.std(xLs)))
			fout.write(",%f, %f, %f,%f" % (0.0, 0.0, np.mean(yLs), np.std(yLs)))
			fout.write(",%f, %f, %f,%f" % (0.0, 0.0, np.mean(zLs), np.std(zLs)))

		fout.write("\n")
		"""
	# Close data file
	fout.close()

	# Prettify
	plt.figure(energyFig.number)
	plt.xlabel("Time (ns)", fontsize=MEDIUM)
	plt.ylabel("Energy (pN.nm)", fontsize=MEDIUM)
	plt.legend(loc=0, fontsize=MEDIUM)
	
	plt.tight_layout()
	oFname = "../results/BSAHydrogel_ff%.3f_Kinetics_Energies.png" % (ff)
	plt.savefig(oFname, dpi=100)
	plt.clf()

	plt.figure(coordFig.number)
	plt.xlabel("Time (ns)", fontsize=MEDIUM)
	plt.ylabel("Coordination", fontsize=MEDIUM)
	plt.legend(loc=0, fontsize=MEDIUM)
	
	plt.tight_layout()
	oFname = "../results/BSAHydrogel_ff%.3f_Kinetics_Coordination.png" % (ff)
	plt.savefig(oFname, dpi=100)
	plt.clf()

	plt.figure(xLFig.number)
	plt.xlabel("Time (ns)", fontsize=MEDIUM)
	plt.ylabel("Length (nm)", fontsize=MEDIUM)
	plt.legend(loc=0, fontsize=MEDIUM)
	
	plt.tight_layout()
	oFname = "../results/BSAHydrogel_ff%.3f_Kinetics_Widths.png" % (ff)
	plt.savefig(oFname, dpi=100)
	plt.clf()

	fIndex += 1
